module constants
  implicit none
  integer::i,j,nt
  integer, parameter :: imax = 100,jmax=imax,ntmax = 2500
  real(8):: T(0:imax,0:jmax),xmax,ymax,x,y,dx,dy,dt,kp
  integer,parameter :: SP = kind(1.0)
  integer,parameter :: DP = selected_real_kind(2*precision(1.0_SP))
end module constants

module print
  use constants
  implicit none
 
 contains
  subroutine print__profile(f)
    real(DP), dimension(0:imax,0:jmax), intent(in) :: f
    integer :: counter = 0
    character(len=*), parameter :: base = "./data/temp.j=middle."
    character(len=4) :: serial_num
    character(len=40) filename
  
    write(serial_num,'(i4.4)') counter

    write(filename,"(a,i5.5,a)") base//serial_num
    open(10,file=filename)
    do j=0,jmax
      do i = 0 ,imax
       write(10,"(2f10.4,2f10.4,2f10.4)") i*dx, j*dx,f(i,j)
      end do
       write(10,"(2f10.4,2f10.4,2f10.4)") 
    end do
    close(10)
    counter = counter + 1
    
  end subroutine print__profile
end module print

program thermal_explicit
  use constants
  use print

  implicit none

  dt   = 0.004
  kp   = 1
  xmax = 10
  ymax=xmax
  dx   = xmax/imax
  dy   = ymax/jmax

  
  open(10,file="initial.txt")
  open(11,file="final.txt")

   !initial state
   do j=0,jmax
     do i = 0, imax
       x = dx*i
       y = dy*j
       if (i < imax/2) then
         T(i,j) = x
       else
         T(i,j) = xmax - x
       endif

      write(10,*) ,x,y,T(i,j)
      end do
    end do
   call print__profile(T(:,:))

   
   !Time step
   do nt = 1, ntmax
     do j=1,jmax-1
       do i = 1, imax-1
          T(i,j) = T(i,j) + kp*dt/(dx*dx)*(T(i+1,j)- 2*T(i,j) +T(i-1,j)) &
                      + kp*dt/(dy*dx)*(T(i,j+1)- 2*T(i,j) +T(i,j-1))
       end do
      end do
      if (mod(nt,100)==0) call print__profile(T(:,:))
      write(*,*)nt
   end do
   
   !Final state
   do j=0,jmax
     do i = 0, imax
       x = dx*i
       y = dy*j
       write(11,*) x,y,T(i,j)
    end do 
   end do
end program thermal_explicit
